@@ -37,11 +37,13 @@ class RedditConfig(BaseSourceConfig):
     lookup_period: Optional[str] = None
     include_post_meta: Optional[bool] = True
     post_meta_field: str = "post_meta"
-    cred_info: RedditCredInfo = Field(RedditCredInfo())
+    cred_info: Optional[RedditCredInfo] = Field(None)
 
     def __init__(self, **data: Any):
         super().__init__(**data)
 
+        self.cred_info = self.cred_info or RedditCredInfo()
+
         self._reddit_client = Reddit(
             client_id=self.cred_info.client_id.get_secret_value(),
             client_secret=self.cred_info.client_secret.get_secret_value(),
